<?php
/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Magento\GoogleShopping\Helper;

/**
 * Google Content Data Helper
 *
 * @author     Magento Core Team <core@magentocommerce.com>
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * Magento string lib
     *
     * @var \Magento\Framework\Stdlib\String
     */
    protected $string;

    /**
     * Store manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\Stdlib\String $string
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Stdlib\String $string,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        $this->string = $string;
        $this->_storeManager = $storeManager;
        parent::__construct($context);
    }

    /**
     * Get Google Content Product ID
     *
     * @param int $productId
     * @param int $storeId
     * @return string
     */
    public function buildContentProductId($productId, $storeId)
    {
        return $productId . '_' . $storeId;
    }

    /**
     * Remove characters and words not allowed by Google Content in title and content (description).
     *
     * To avoid "Expected response code 200, got 400.
     * Reason: There is a problem with the character encoding of this attribute"
     *
     * @param string $string
     * @return string
     */
    public function cleanAtomAttribute($string)
    {
        return $this->string->substr(preg_replace('/[\pC¢€•—™°½]|shipping/ui', '', $string), 0, 3500);
    }

    /**
     * Normalize attribute's name.
     * The name has to be in lower case and the words are separated by symbol "_".
     * For instance: Meta Description = meta_description
     *
     * @param string $name
     * @return string
     */
    public function normalizeName($name)
    {
        return strtolower(preg_replace('/[\s_]+/', '_', $name));
    }

    /**
     * Parse \Exception Response Body
     *
     * @param string $message \Exception message to parse
     * @param null|\Magento\Catalog\Model\Product $product
     * @return string
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function parseGdataExceptionMessage($message, $product = null)
    {
        $result = [];
        foreach (explode("\n", $message) as $row) {
            if (trim($row) == '') {
                continue;
            }

            if (strip_tags($row) == $row) {
                $row = preg_replace('/@ (.*)/', __('See \'\1\''), $row);
                if ($product !== null) {
                    $row .= ' ' . __(
                        "for product '%1' (in '%2' store)",
                        $product->getName(),
                        $this->_storeManager->getStore($product->getStoreId())->getName()
                    );
                }
                $result[] = $row;
                continue;
            }

            // parse not well-formatted xml
            preg_match_all('/(reason|field|type)=\"([^\"]+)\"/', $row, $matches);

            if (is_array($matches) && count($matches) == 3
                && is_array($matches[1]) && count($matches[1]) > 0
            ) {
                $c = count($matches[1]);
                for ($i = 0; $i < $c; $i++) {
                    if (isset($matches[2][$i])) {
                        $result[] = ucfirst($matches[1][$i]) . ': ' . $matches[2][$i];
                    }
                }
            }
        }
        return implode(". ", $result);
    }
}
