<?php
/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Magento\Tools\Di\App\Task\Operation;

use Magento\Tools\Di\App\Task\OperationInterface;

class InterceptionCache implements OperationInterface
{
    /**
     * @var array
     */
    private $data = [];

    /**
     * @var \Magento\Framework\Interception\Config\Config
     */
    private $configInterface;

    /**
     * @var \Magento\Tools\Di\Code\Reader\Decorator\Interceptions
     */
    private $interceptionsInstancesNamesList;

    /**
     * @param \Magento\Framework\Interception\Config\Config $configInterface
     * @param \Magento\Tools\Di\Code\Reader\Decorator\Interceptions $interceptionsInstancesNamesList
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Interception\Config\Config $configInterface,
        \Magento\Tools\Di\Code\Reader\Decorator\Interceptions $interceptionsInstancesNamesList,
        array $data = []
    ) {
        $this->configInterface = $configInterface;
        $this->interceptionsInstancesNamesList = $interceptionsInstancesNamesList;
        $this->data = $data;
    }

    /**
     * Flushes interception cached configuration and generates a new one
     *
     * @return void
     */
    public function doOperation()
    {
        if (empty($this->data)) {
            return;
        }

        $definitions = [];
        foreach ($this->data as $path) {
            $definitions = array_merge($definitions, $this->interceptionsInstancesNamesList->getList($path));
        }

        $this->configInterface->initialize($definitions);
    }
}
